package org.medias.eccad.metier.statistique;

import java.awt.Color;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.text.DecimalFormat;
import java.text.NumberFormat;
import java.text.ParseException;
import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Date;
import java.util.List;

import org.jfree.chart.ChartFactory;
import org.jfree.chart.ChartUtilities;
import org.jfree.chart.JFreeChart;
import org.jfree.chart.axis.DateAxis;
import org.jfree.chart.axis.NumberAxis;
import org.jfree.chart.axis.NumberTickUnit;
import org.jfree.chart.plot.XYPlot;
import org.jfree.chart.renderer.xy.XYErrorRenderer;
import org.jfree.data.time.FixedMillisecond;
import org.jfree.data.time.TimeSeries;
import org.jfree.data.time.TimeSeriesCollection;
import org.jfree.data.xy.XYDataset;
import org.medias.eccad.helpers.Donnees;
import org.medias.eccad.helpers.LoggerPerso;
import org.medias.eccad.modele.Valeur;

public class ServiceGraphe {
	
	public String genereGraphe(String identifiant, List<Valeur> liste_valeur, String param, String produit, String unite, String secteur, String rcp, String alt) {
		File fichier = new File(Donnees.getHome() + "/images/graphes_temporaires/" + identifiant +"_graphe.png");
		OutputStream out = null;
		
		try {
			out = new FileOutputStream(fichier);
			getGraphe(out, liste_valeur, param, produit, unite, secteur, rcp, alt);
		} catch (FileNotFoundException e) {
			LoggerPerso.log(ServiceGraphe.class, LoggerPerso.ERROR, "erreur d'écriture du graphe", e);
			return "/";
		} catch (IOException e) {
			LoggerPerso.log(ServiceGraphe.class, LoggerPerso.ERROR, "erreur d'écriture du graphe", e);
			return "/";
		}
		
		return "/images/graphes_temporaires/" + identifiant +"_graphe.png";
	}
	
	public void getGraphe(OutputStream out, List<Valeur> liste_valeur, String param, String produit, String unite, String secteur, String rcp, String alt) 
    throws IOException { 
    
		// CategoryDataset dataset = new DefaultCategoryDataset(); 
		TimeSeries time = new TimeSeries("");
		double min = Double.MAX_VALUE, max = -1;
		DecimalFormat df = new DecimalFormat("0.##E0");
		try {
			Date dateVal;
			String datePrec = "";
			ArrayList<Double> vals = new ArrayList<Double>();
			int nbGrilles = -1;
			double val;
			for (Valeur valeur : liste_valeur) {
				val = Double.parseDouble(valeur.getValeur());
				if (!valeur.getNom().equals(datePrec)) {
					nbGrilles++;
					vals.add(Double.valueOf(val));
//					val = Double.parseDouble(valeur.getValeur());
				} else {
					vals.set(nbGrilles, Double.valueOf(vals.get(nbGrilles).doubleValue()+val));
//					val += Double.parseDouble(valeur.getValeur());
				}
//  	  			LoggerPerso.log(ServiceGraphe.class, LoggerPerso.DEBUG, "valeur graphe="+valeur.getValeur()+", nom="+valeur.getNom()+", val="+val+", format="+df.format(val));
				dateVal = (new SimpleDateFormat("yyyyMMdd")).parse(valeur.getNom());
				time.addOrUpdate(new FixedMillisecond(dateVal),vals.get(nbGrilles).doubleValue());
				datePrec = valeur.getNom();
//  		  	time.add(new Day(Integer.parseInt(valeur.getNom().substring(6, 8)), 
//						 Integer.parseInt(valeur.getNom().substring(4, 6)), 
//						 Integer.parseInt(valeur.getNom().substring(0, 4))), 
//				Double.parseDouble(valeur.getValeur()));
			}
			for (int num = 0; num < vals.size(); num++) {
				val = vals.get(num).doubleValue();
				if (min > val) min = val;
				if (max < val) max = val;
			}
		} catch (ParseException e) {
			e.printStackTrace();
		}
    
		XYDataset dataset = new TimeSeriesCollection(time);
		//dataset.setDomainIsPointsInTime(true);
		String titre = param;
		if (produit.equals("IPCC")) {
			titre += ", "+secteur;
			if (!rcp.equals("None")) {
				titre += ", RCP "+rcp;
			}
		}
		titre += " : "+produit;
		if (produit.equals("IPCC") && !alt.equals("0")) {
			if (alt.equals("Sum"))
				titre += " (Sum altitude)";
			else
				titre += " ("+alt+" km)";
		}
		JFreeChart barChart = ChartFactory.createTimeSeriesChart(titre, "Date", 
				unite, dataset, false, true, false);
		XYPlot plot = barChart.getXYPlot();
		DateAxis axis = (DateAxis) plot.getDomainAxis();
		axis.setDateFormatOverride(new SimpleDateFormat("yyyy-MM-dd"));
		plot.setRenderer(new XYErrorRenderer());
		plot.getRenderer().setSeriesPaint(0, Color.blue);
//		if (!unite.equals("pers/km2") && !unite.equals("%") && !unite.equals("km2") && !unite.equals("number of fires")) {
//			plot.getRangeAxis().setStandardTickUnits(new StandardTickUnitSource());
//		}
//			plot.getRangeAxis().setUpperBound(max);
//			plot.getRangeAxis().setLowerBound(min);
		if (max != -1) {
			NumberAxis na = (NumberAxis)plot.getRangeAxis();
			na.setAutoTickUnitSelection(false);
			if (max == min && min != 0) {
				min = 0;
			} else if (max == min && min == 0) {
				max = 6;
			}
			na.setRangeWithMargins(min-(max-min)/20,max+(max-min)/20);
			if (!unite.equals("pers/km2") && !unite.equals("%") && !unite.equals("km2") && !unite.equals("number of fires")) {
				na.setTickUnit(new NumberTickUnit((max-min)/6,df));
			} else {
				na.setTickUnit(new NumberTickUnit((max-min)/6,NumberFormat.getIntegerInstance()));
			}
		}
//  	plot.setBackgroundPaint(Color.white);

//		plot.setRenderer(new StandardXYItemRenderer(
//                StandardXYItemRenderer.LINES + StandardXYItemRenderer.SHAPES));

		/*plot.setRenderer(new XYBarRenderer());
    	svgTest(barChart, "XYBarRenderer");
    	plot.setRenderer(new XYStepRenderer());
    	svgTest(barChart, "XYStepRenderer");
    	plot.setRenderer(new ClusteredXYBarRenderer());
    	svgTest(barChart, "ClusteredXYBarRenderer");
    	plot.setRenderer(new CyclicXYItemRenderer());
		svgTest(barChart, "CyclicXYItemRenderer");
    	plot.setRenderer(new DefaultXYItemRenderer());
		svgTest(barChart, "DefaultXYItemRenderer");
    	plot.setRenderer(new DeviationRenderer());
		svgTest(barChart, "DeviationRenderer");
    	plot.setRenderer(new HighLowRenderer());
		svgTest(barChart, "HighLowRenderer");
    	plot.setRenderer(new StandardXYItemRenderer());
		svgTest(barChart, "StandardXYItemRenderer");
    	plot.setRenderer(new VectorRenderer());
		svgTest(barChart, "VectorRenderer");
    	plot.setRenderer(new XYAreaRenderer());
		svgTest(barChart, "XYAreaRenderer");
    	plot.setRenderer(new XYAreaRenderer2());
		svgTest(barChart, "XYAreaRenderer2");
    	plot.setRenderer(new XYBarRenderer());
		svgTest(barChart, "XYBarRenderer");
    	plot.setRenderer(new XYBlockRenderer());
		svgTest(barChart, "XYBlockRenderer");
    	plot.setRenderer(new XYBubbleRenderer());
    	svgTest(barChart, "XYBubbleRenderer");
    	plot.setRenderer(new XYDifferenceRenderer());
		svgTest(barChart, "XYDifferenceRenderer");
    	plot.setRenderer(new XYDotRenderer());
		svgTest(barChart, "XYDotRenderer");
    	plot.setRenderer(new XYErrorRenderer());
		svgTest(barChart, "XYErrorRenderer");
    	plot.setRenderer(new XYLine3DRenderer());
		svgTest(barChart, "XYLine3DRenderer");
    	plot.setRenderer(new XYLineAndShapeRenderer());
		svgTest(barChart, "XYLineAndShapeRenderer");
    	plot.setRenderer(new XYStepAreaRenderer());
		svgTest(barChart, "XYStepAreaRenderer");
    	plot.setRenderer(new XYStepRenderer());
		svgTest(barChart, "XYStepRenderer");
    	plot.setRenderer(new YIntervalRenderer());
    	svgTest(barChart, "YIntervalRenderer");*/

		ChartUtilities.writeChartAsPNG(out, barChart, 720, 360);
	}
	
	/*private void svgTest(JFreeChart graphe, String type) {
		File fichier = new File("/tmp/" + type + ".png");
		OutputStream out = null;
		try {
			out = new FileOutputStream(fichier);
			ChartUtilities.writeChartAsPNG(out, graphe, 720, 360);
			out.close();
		} catch (FileNotFoundException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		} catch (IOException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}*/
}
